#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include "mt64.h"
#include "basic_functions.h"
#include "percolating_clusters.h"



#define SENS 1.0e-6

//////////////////////////////////////









double inner_product (int N, double *vec1, double *vec2)
{
  double n1 = normalize_vector(N, vec1);
  double n2 = normalize_vector(N, vec2);
  double inner = 0.0;
  int i;
  for(i=1;i<=N;i++) inner += vec1[i]*vec2[i];
  return inner;
}


double pearson_coefficient (int N, double *vec1, double *vec2)
{
  int i;
  double a, b, c, av1, av2;
  av1 = av2 = 0.0;
  for(i=1;i<=N;i++)
    {
      av1 += vec1[i];
      av2 += vec2[i];
    }
  av1 /= (double)N;
  av2 /= (double)N;
  
  a = b = c = 0.0;
  for(i=1;i<=N;i++)
    {
      a += (vec1[i]-av1)*(vec2[i]-av2);
      b += (vec1[i]-av1)*(vec1[i]-av1);
      c += (vec2[i]-av2)*(vec2[i]-av2);
    }

  return a/sqrt(b)/sqrt(c);

}





double simple_estimation (int N, int **bond)
{
  int i;
  double first, second;
  first = second = 0.0;
  for(i=1;i<=N;i++)
    {
      first += (double)bond[0][i];
      second += (double)bond[0][i]*(double)bond[0][i];;
    }
  return (second-first)/first;
}






void from_bond_to_edges (int N, int **bond, int **edges)
{

  int i, j, n;
  int T = 0;
  for(i=1;i<=N;i++) T +=bond[0][i];
  T = T / 2;
  edges[0] = (int *)realloc(edges[0], (T+1)*sizeof(int));
  edges[1] = (int *)realloc(edges[1], (T+1)*sizeof(int));
  edges[0][0] = 0;
  for(i=1;i<=N;i++)
    {
      for(j=1;j<=bond[0][i];j++)
        {
          n = bond[i][j];
          if(i<n)
            {
              edges[0][0]++;
              //printf("%d %d\n",edges[0][0],T);                                                                                          
              //fflush(stdout);                                                                                                           
              edges[0][edges[0][0]]=i;
              edges[1][edges[0][0]]=n;
            }
        }
    }

}


int count_nodes (char *filename)
{
  FILE *f;
  int i, j, q;
  int N = 0;

  f = fopen(filename,"r");
  while(!feof(f))
    {
      q = fscanf(f,"%d %d",&i,&j);
      if (q<=0) goto exit_file;
      if(i>N) N = i;
      if(j>N) N = j;
    }
 exit_file:
  fclose(f);
  return N;
}


void read_edges (char *filename, int N, int **bond)
{
  FILE *f;
  int i, j, q;

  for(i=1;i<=N;i++) bond[0][i] = 0;

  f = fopen(filename,"r");
  while(!feof(f))
    {
      q = fscanf(f,"%d %d",&i,&j);
      if (q<=0) goto exit_file_A;

      //add connection                                                                                                                   
      bond[0][i]++;
      bond[0][j]++;
      
    }
 exit_file_A:
  fclose(f);


  for(i=1;i<=N;i++)
    {
      if(bond[0][i]<1) 
	{
	  printf("## %d %d\n",i,bond[0][i]);
	  //exit(0);
	}
      bond[i] = (int *)realloc(bond[i], (bond[0][i]+1)*sizeof(int));
      bond[0][i] = 0;
    }



  f = fopen(filename,"r");
  while(!feof(f))
    {
      q = fscanf(f,"%d %d",&i,&j);
      if (q<=0) goto exit_file;
      
      //add connection
      bond[0][i]++;
      bond[i][bond[0][i]] = j;

      bond[0][j]++;
      bond[j][bond[0][j]] = i;

    }
 exit_file:
  fclose(f);
}

















int largest_connected_component (int N, int **bond, int **tmp_bond)
{
  int i, j, k, s, n;
  
  int *comp=(int *)malloc((N+1)*sizeof(int));
  int *vec1=(int *)malloc((N+1)*sizeof(int));
  int *vec2=(int *)malloc((N+1)*sizeof(int));
  for(i=0;i<=N;i++)  comp[i]=-1;
  comp[0] = 0;


  
  for(n=1;n<=N;n++)
    {

      if(comp[n]<0){

	s=n;

	comp[0]++;
	comp[s] = comp[0];

	vec1[0] = 1;
	vec1[1] = s;


	while(vec1[0]>0) {

	  //printf("#c %d\n",comp[0]);
	  
	  vec2[0]=0;
	  for(i=1;i<=vec1[0];i++)
	    {
	      s=vec1[i];
	     

	      for(j=1;j<=bond[0][s];j++) 
		{
		  k=bond[s][j];
		  if(comp[k]<0) 
		    {
		      vec2[0]++;
		      vec2[vec2[0]]=k;
		      comp[k]=comp[0];
		    }
		}

	    }
	
      

	vec1[0]=vec2[0];
	if(vec2[0]>0) for(i=1;i<=vec2[0];i++) vec1[i]=vec2[i];
	}
      }
    }


  //printf("#comp %d  %d\n\n\n\n",comp[0],N);


  int *num = (int *)malloc((comp[0]+1)*sizeof(int));
  for(i=0;i<=comp[0];i++) num[i] = 0;
  for(i=1;i<=N;i++) num[comp[i]]++;

  int max = 0;
  int cmax;

  for(i=1;i<=comp[0];i++)
    {
      if(num[i]>max) 
	{
	  max = num[i];
	  cmax = i;
	}
    }

  
  int *index = (int *)malloc((N+1)*sizeof(int));
  index[0] = 0;
  for(i=1;i<=N;i++)
    {
      if(comp[i]==cmax) 
	{
	  index[0]++;
	  index[i] = index[0];
	}
      else{
	index[i] = -1;
      }
    }


  for(i=1;i<=N;i++) tmp_bond[0][i]=0;

  
  for(i=1;i<=N;i++)
    {
      if(index[i] > 0){
	for(j=1;j<=bond[0][i];j++)
	  {
	    
	    k = index[i];
	    s = index[bond[i][j]];

	    tmp_bond[0][k] ++;
	    tmp_bond[k] =  (int *)realloc(tmp_bond[k],(tmp_bond[0][k]+1)*sizeof(int));
	    tmp_bond[k][tmp_bond[0][k]] = s;	  
	  }
      }
    }
      
 



  free(comp);
  free(vec1);
  free(vec2);
  free(num);
  free(index);
 


  return max;

}








////////////////////



void init_vector (int N, double *vec)
{
  int i;
  for(i=1;i<=N;i++) vec[i]=-1.0+2.0*genrand64_real3();
  //for(i=1;i<=N;i++) vec[i]=genrand64_real3();
  double q=normalize_vector (N, vec);
}

double normalize_vector (int N, double *vec)
{
  int i;
  double norm=0.0;
  for(i=1;i<=N;i++) norm+=vec[i]*vec[i];
  norm=sqrt(norm);
  for(i=1;i<=N;i++) vec[i]/=norm;
  return norm;
}

double estimate_error (int N, double *vec1, double *vec2)
{
  double err=0.0;
  int i;
  //for(i=1;i<=N;i++) err += fabs(fabs(vec1[i])-fabs(vec2[i]));
  for(i=1;i<=N;i++) if(fabs(fabs(vec1[i])-fabs(vec2[i])) > err) err = fabs(fabs(vec1[i])-fabs(vec2[i])) ;
  return err;
}


double principal_nonbacktracking (int N, int **bond, double *eigenvector)
{

  int i, j, n;
  double l2, err, tmp;

  double *vec = (double *)malloc((2*N+1)*sizeof(double));
  double *vec_tmp = (double *)malloc((2*N+1)*sizeof(double));
  

  
  init_vector (2*N, vec);
 

  

  int iter=0;
  err=1.0e10;
  while(err>SENS)
  {
    iter++;
    l2 = single_iteration_nonback (N, bond, vec, vec_tmp);
    err = estimate_error (2*N, vec, vec_tmp);
    if(iter>10000) 
      {
	printf("#A) Algorithm did not converge!!\n");
	l2 = -100.0;
	goto exitloop;
      }
   }


 exitloop:
  for(i=1;i<=N;i++) eigenvector[i] = fabs(vec[i]);



  free(vec);
  free(vec_tmp);



  return l2;

}





double single_iteration_nonback(int N, int **bond, double *vec, double *vec_tmp)
{
  double tmp;
  int i, j, n;

  

  for(i=1;i<=2*N;i++) 
    {
      vec_tmp[i] = vec[i];
      vec[i] = 0.0;
    }


  for(i=1;i<=N;i++) 
    {
      for(j=1;j<=bond[0][i];j++)
	{
	  n = bond[i][j];
	  tmp = 1.0;
	  if(n!=i) vec[n] += tmp*vec_tmp[i];
	  if(n==i) vec[n] += 2.0*tmp*vec_tmp[i];
	}
    }

  for(i=1;i<=N;i++)
    {
      vec[i] += vec_tmp[i+N] * (1.0 - (double)bond[0][i]); 
    }
 
  for(i=N+1;i<=2*N;i++) vec[i] = vec_tmp[i - N];
  



  double l2=normalize_vector (2*N, vec);
 
  return l2;
}








//////////////////////////




double principal_adjacency (int N, int **bond, double *eigenvector)
{
  int i;
  double *vec = (double *)malloc((N+1)*sizeof(double));
  double *vec_tmp = (double *)malloc((N+1)*sizeof(double));
  double l2, err, oldl2;



  init_vector (N, vec);


  oldl2 = 10000;
  l2 = 0.0;

  int iter=0;
  err=1.0e10;
  while(err>SENS)
    {
      iter++;
      l2 = single_iteration_adjacency (N, bond, vec, vec_tmp);
      err = estimate_error (N, vec, vec_tmp);
      //printf("%d %g %g\n",iter,err,l2);
      if(iter>10000)
	{
	  //printf("#B) Algorithm did not converge!!\n");
	  //l2 = -100.0;
	  goto exitloop;
	}
      oldl2 = l2;
    }


 exitloop:
  for(i=1;i<=N;i++) eigenvector[i] = fabs(vec[i]);



  free(vec);
  free(vec_tmp);
 


  return l2;
}



double single_iteration_adjacency (int N, int **bond, double *vec, double *vec_tmp)
{

  int i, j, n;
 

  for(i=1;i<=N;i++)
    {
      vec_tmp[i]= vec[i];
      vec[i] = 0.0; 
    }

  for(i=1;i<=N;i++)
    {
      for(j=1;j<=bond[0][i];j++)
	{
	  n = bond[i][j];
	  vec[n] += vec_tmp[i];
	}
    }


  double l2=normalize_vector (N, vec);
  return l2;

}












///////////////////////////////////

double principal_nonbacktracking_true (int N, int **bond, double *eigenvector)
{
  int i;
  double l;
  
  int M = 0;
  for(i=1;i<=N;i++) M+=bond[0][i];

  //memory allocation
  int **reference = (int **)malloc((N+1)*sizeof(int *));
  for(i=1;i<=N;i++) reference[i] = (int *)malloc((bond[0][i]+1)*sizeof(int));
  double *vec = (double *)malloc((M+1)*sizeof(double));
  int **ref_vec = (int **)malloc((M+1)*sizeof(int *));
  int **bond_non = (int **)malloc((M+1)*sizeof(int *));
  bond_non[0] = (int *)malloc((M+1)*sizeof(int));
  int **trasp_bond = (int **)malloc((M+1)*sizeof(int *));
  trasp_bond[0] = (int *)malloc((M+1)*sizeof(int));
  for(i=1;i<=M;i++)
    {
      ref_vec[i] = (int *)malloc(2*sizeof(int));
      bond_non[0][i] = 0;
      bond_non[i] = (int *)malloc(1*sizeof(int));
      trasp_bond[0][i] = 0;
      trasp_bond[i] = (int *)malloc(1*sizeof(int));
    }
  /////


  construct_non_backtracking_matrix (N, bond, reference, M, bond_non, ref_vec);
  l = principal_adjacency (M, bond_non, vec);

  //matrix_transpose (M, bond_non, trasp_bond);
  //l = principal_adjacency (M, trasp_bond, vec);

  
  for(i=1;i<=N;i++) eigenvector[i] = 0.0;

  for(i=1;i<=M;i++)
    {
      eigenvector[ref_vec[i][1]] += fabs(vec[i]);
      //eigenvector[ref_vec[i][0]] += fabs(vec[i]);
    }



  //memory release
  free(vec);
  for(i=0;i<=M;i++) free(bond_non[i]);
  free(bond_non);
  for(i=0;i<=M;i++) free(trasp_bond[i]);
  free(trasp_bond);
  for(i=1;i<=M;i++) free(ref_vec[i]);
  free(ref_vec);
  for(i=1;i<=N;i++) free(reference[i]);
  free(reference);
  ///

  return l;
}



void construct_non_backtracking_matrix (int N, int **bond, int **reference, int M, int **bond_non, int **ref_vec)
{
  int i, j, n, m, m1, m2, q, s;

  m = 0;
  for(i=1;i<=N;i++)
    {
      for(j=1;j<=bond[0][i];j++)
	{
	  n = bond[i][j];
	  m++;
	  ref_vec[m][0] = i;
	  ref_vec[m][1] = n;
	  reference[i][j] = m;
	}
    }



  for(i=1;i<=N;i++)
    {
      for(j=1;j<=bond[0][i];j++)
	{
	  n = bond[i][j];
	  m1 = reference[i][j];
	  for(q=1;q<=bond[0][n];q++)
	    {
	      s = bond[n][q];
	      if(s!=i)
		{
		  m2 = reference[n][q];
		  bond_non[0][m1]++;
		  bond_non[m1] = (int *)realloc(bond_non[m1],(bond_non[0][m1]+1)*sizeof(int));
		  bond_non[m1][bond_non[0][m1]] = m2;
		}
	    }
	}
    }
}



void matrix_transpose (int N, int **bond, int **trasp_bond)
{

  int i, j, n;

  for(i=1;i<=N;i++)
    {
      for(j=1;j<=bond[0][i];j++) 
	{
	  n = bond[i][j];

	  trasp_bond[0][n]++;
	  trasp_bond[n] = (int *)realloc(trasp_bond[n], (trasp_bond[0][n]+1)*sizeof(int));
	  trasp_bond[n][trasp_bond[0][n]] = i;
	}
    }




}




























