#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include "mt64.h"
#include "basic_functions.h"
#include "percolating_clusters.h"
#include "new_theory.h"


int main (int argc, char **argv)
{

  int i, j;

  init_genrand64((unsigned) time(NULL) * getpid());



  int NA = count_nodes (argv[1]);
  int NB = count_nodes (argv[2]);

  int N = NA;
  if (NB>NA) N = NB;

  printf("#Number of nodes %d\n",N);



  FILE *f, *g;
  g = fopen("log.log","w");


  fprintf(g,"#Total nodes in A %d\n",NA);
  fprintf(g,"#Total nodes in B %d\n",NB);




  //memory allocation
  int **bondA = (int **)malloc((N+1)*sizeof(int *));
  bondA[0] = (int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++) bondA[i] = (int *)malloc(1*sizeof(int));
  //////////




  //memory allocation
  int **bondB = (int **)malloc((N+1)*sizeof(int *));
  bondB[0] = (int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++) bondB[i] = (int *)malloc(1*sizeof(int));
  //////////



  //memory allocation                                                                                                                       
  int **onlyA = (int **)malloc((N+1)*sizeof(int *));
  onlyA[0] = (int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++) 
    {
      onlyA[0][i] =0;
      onlyA[i] = (int *)malloc(1*sizeof(int));
    }

  int **onlyB = (int **)malloc((N+1)*sizeof(int *));
  onlyB[0] = (int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++) 
    {
      onlyB[0][i]=0;
      onlyB[i] = (int *)malloc(1*sizeof(int));
    } 

  int **intersection = (int **)malloc((N+1)*sizeof(int *));
  intersection[0] = (int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++)
    {
      intersection[0][i]=0;
      intersection[i] = (int *)malloc(1*sizeof(int));
    }
 //////////   





  read_edges (argv[1], N, bondA);
  read_edges (argv[2], N, bondB);



 
  int A, B, I, OA, OB;

  find_intersection (N, bondA, bondB, intersection, onlyA, onlyB);
  I = OA = OB = A = B = 0;
  for(i=1;i<=N;i++)
    {
      A += bondA[0][i];
      B += bondB[0][i];
      I += intersection[0][i];
      OA += onlyA[0][i];
      OB += onlyB[0][i];
    }
  fprintf(g,"#Edges\tA %d\tB %d\n",A,B);
  fprintf(g,"#Intersection %d\tA %d\tB %d\n",I,OA,OB);


  double res[2];
  find_transition_theoretical_model_infection_nodes (N, intersection, onlyA, onlyB, res);
  fprintf(g,"#Predictions %g %g\n",res[0],res[1]);

  find_transition_point_new_theory (N, onlyA, onlyB, intersection, res);
  fprintf(g,"#NewPredictions %g %g\n",res[0],res[1]);

  fclose(g);







  //
  //
  //Begin: Percolation phase diagram from numerical simulations
  //
  //
  //

  int t, T;
  T = 10000;
  double val, ord, susc, p, dp;
  
  dp = 1.0e-2;
  
  
  f = fopen("numerical.dat","w");
  p = dp;
  while(p<1.0){
    ord = susc = 0.0;
    for(t=1;t<=T;t++){
      val = mutually_connected_largest (N, bondA, bondB, p) ;
      ord += (double)val / (double)N;
      susc += (double)val / (double)N * (double)val / (double)N;;
    }
    ord /= (double)T;
    susc /= (double)T;
    susc = (susc - ord*ord) * (double)N;
    
    
    
    printf("%g %g %g\n",p,ord, susc);
    
   fprintf(f,"%g %g %g\n",p , ord, susc);
   p += dp;
  }
  
  fclose(f);
  //
  //
  //End: Percolation phase diagram from numerical simulations
  //
  //
  



  
  //
  //
  //Begin: Percolation phase diagram approximated according to the approach of
  //Percolation in real interdependent networks 
  //F. Radicchi 
  //Nature Phys. 11, 597-602 (2015) 
  //
  //
  //
  ///////////////////////////
  int V = 1000;
  double dx = 1.0/(double)V;
  double **order = (double **)malloc((V+1)*sizeof(double *));
  for(i=0;i<=V;i++) 
    {
      order[i] = (double *)malloc(2*sizeof(double));
      order[i][0] = (double)i*dx;
      order[i][1] = 0.0;
    }
  

  theoretical_model_infection_nodes (N, intersection, onlyA, onlyB, order, V);
  f = fopen("theory.dat","w");
  for(i=0;i<=V;i++) fprintf(f,"%g %g\n",order[i][0],order[i][1]);
  fclose(f);
  //
  //
  //End: Percolation phase diagram approximated according to the approach of
  //Percolation in real interdependent networks 
  //F. Radicchi 
  //Nature Phys. 11, 597-602 (2015) 
  //
  //
  //


 for(i=0;i<=V;i++) free(order[i]);
 free(order);


  



 ///////////////////////////////
 //
 //
 //
 //
 //Begin: Percolation phase diagram approximated according to the approach of
 //Percolation in real multiplex networks 
 //G. Bianconi and F. Radicchi 
 //arXiv:1610.08708 (2016) 
 //
 //
 //



  order_parameter_new_theory (N, onlyA, onlyB, intersection, "theory_novel.dat");



  //
  //
  //
  //
  //End: Percolation phase diagram approximated according to the approach of
  //Percolation in real multiplex networks 
  //G. Bianconi and F. Radicchi 
  //arXiv:1610.08708 (2016) 
  //
  //
  //

 //////////////////////////////










  //memory release
  for(i=0;i<=N;i++) free(bondA[i]);
  free(bondA);

  for(i=0;i<=N;i++) free(bondB[i]);
  free(bondB);

  for(i=0;i<=N;i++) free(intersection[i]);
  free(intersection);
  for(i=0;i<=N;i++) free(onlyA[i]);
  free(onlyA);
  for(i=0;i<=N;i++) free(onlyB[i]);
  free(onlyB);
  //////////////////




  return 0;
}















